﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml;
using System.Xml.Linq;
using System.Net;

namespace Client
{
    class Program
    {
        static void Main(string[] args)
        {
            // navigate to the root of the service
            Console.WriteLine("Navigating to the root of the service...");
            Uri uri = new Uri("http://localhost:63965/");
            CookieContainer cookies = new CookieContainer();
            var doc = XDocument.Load(uri.ToString());
            doc.AddAnnotation(uri);
          
            // navigate to public bookmarks
            Console.WriteLine("Navigating to public bookmarks...");
            var links = doc.Body().Ul("nav-links").Lis();
            var bookmarksLink = links.Where(l => l.HasAnchor("public-bookmarks-link")).First();
            var bookmarksDoc = bookmarksLink.Anchor().Navigate();
            bookmarksDoc.AddAnnotation(cookies);

            // display list of bookmarks
            Console.WriteLine("\nPublic bookmarks found in the system:");
            var bookmarks = bookmarksDoc.Body().Ol("bookmark-list").Lis();
            bookmarks.Each(bm => Console.WriteLine("{0}: {1}",
                bm.Anchor().AnchorText, bm.Anchor().AnchorLink));
            Console.WriteLine();

            // navigate to the first bookmark in the list
            Console.WriteLine("Navigating to the first bookmark in the list...");
            var bookmarkDoc = bookmarks.First().Anchor().Navigate();
            var bookmarkDetails = bookmarkDoc.Body().Struct("bookmark");

            // print the bookmark details out to the console window
            Console.WriteLine("Bookmark details:");
            Console.WriteLine("bookmark-id: {0}", bookmarkDetails["bookmark-id"].Value);
            Console.WriteLine("bookmark-url-link: {0}", bookmarkDetails["bookmark-url-link"].Value);
            Console.WriteLine("bookmark-title: {0}", bookmarkDetails["bookmark-title"].Value);
            Console.WriteLine("bookmark-shared: {0}", bookmarkDetails["bookmark-shared"].Value);
            Console.WriteLine("bookmark-last-modified: {0}", bookmarkDetails["bookmark-last-modified"].Value);

            // retrieving login form
            Console.WriteLine("\nRetrieving login form...");
            Uri logonUri = new Uri("http://localhost:63965/Account/Logon");
            var logonDoc = XDocument.Load(logonUri.ToString());
            logonDoc.AddAnnotation(logonUri);
            logonDoc.AddAnnotation(cookies);

            // logging on as skonnard
            Console.WriteLine("Logging in as 'skonnard'");
            var logonForm = logonDoc.Body().Form("account-logon-form");
            logonForm["username"] = "skonnard";
            logonForm["password"] = "password";
            logonForm.Submit();
            Console.WriteLine("Login successful!");

            // create a new bookmark as 'skonnard'
            var createBookmarkDoc = bookmarksDoc.Body().Anchor("create-bookmark-form-link").Navigate();
            createBookmarkDoc.AddAnnotation(cookies);
            var createBookmarkForm = createBookmarkDoc.Body().Form("create-bookmark-form");
            createBookmarkForm["bookmark-title"] = "Test from console!";
            createBookmarkForm["bookmark-url"] = "http://live.com/";
            createBookmarkForm["bookmark-tags"] = "Microsoft, Search";
            createBookmarkForm.Submit();
            Console.WriteLine("\nBookmark created!");
        }
    }
}
