﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;

namespace MvcBookmarkService.Models
{
    public class BookmarksRepository
    {
        BookmarksModelDataContext db = new BookmarksModelDataContext();

        // query methods

        public IQueryable<Bookmark> FindAllBookmarks()
        {
            return db.Bookmarks;
        }

        public IQueryable<Bookmark> FindAllPublicBookmarks()
        {
            return from bm in db.Bookmarks
                   where bm.Shared
                   orderby bm.LastModified descending 
                   select bm;
        }

        public IQueryable<Bookmark> FindBookmarksByUser(string username)
        {
            return from bm in db.Bookmarks
                   where bm.Username.ToLower().Equals(username.ToLower())
                   orderby bm.LastModified descending
                   select bm;
        }

        public IQueryable<Bookmark> FindPublicBookmarksByUser(string username)
        {
            return from bm in db.Bookmarks
                   where bm.Username.ToLower().Equals(username.ToLower()) && bm.Shared
                   orderby bm.LastModified descending
                   select bm;
        }

        public IQueryable<Bookmark> FindBookmarksByTag(string tag)
        {
            return from bm in db.Bookmarks
                   where bm.Tags.ToLower().Contains(tag.ToLower()) && bm.Shared
                   orderby bm.LastModified descending
                   select bm;
        }

        public Bookmark FindBookmarkById(int id)
        {
            return db.Bookmarks.SingleOrDefault(bm => bm.BookmarkID == id);
        }

        // insert/delete methods

        public void AddBookmark(Bookmark bm)
        {
            db.Bookmarks.InsertOnSubmit(bm);
        }

        public void DeleteBookmark(Bookmark bm)
        {
            db.BookmarkTags.DeleteAllOnSubmit(bm.BookmarkTags);
            db.Bookmarks.DeleteOnSubmit(bm);
        }

        public IQueryable<string> FindUsers()
        {
            return (from u in db.Bookmarks
                    orderby u.Username
                    select u.Username).Distinct();
        }

        public IQueryable<Tag> FindTags()
        {
            return from t in db.Tags
                   select t;
        }

        public Tag FindTag(string tag)
        {
            return db.Tags.SingleOrDefault(t => t.Name.Equals(tag.ToLower()));
        }

        public void AddTag(Tag t)
        {
            db.Tags.InsertOnSubmit(t);
        }

        public void AddTagForBookmark(string tagText, Bookmark bm)
        {
            string t = tagText.Trim();
            Tag tag = FindTag(t);
            if (tag == null)
            {
                tag = new Tag();
                tag.Name = t;
                db.Tags.InsertOnSubmit(tag);
                db.SubmitChanges();
            }

            BookmarkTag bmtag = bm.BookmarkTags.FirstOrDefault(bt => bt.TagID == tag.TagID);
            if (bmtag == null)
            {
                bmtag = new BookmarkTag();
                bmtag.BookmarkID = bm.BookmarkID;
                bmtag.TagID = tag.TagID;
                db.BookmarkTags.InsertOnSubmit(bmtag);
            }
        }

        // persistence

        public void Save()
        {
            db.SubmitChanges();
        }
    }
}
